@@ -34,6 +34,11 @@ VIBER_WELCOME_TEXT = 'Hello'
 VIBER_SUCCESS_ADDED_TEXT = 'Success'
 VIBER_FAILED_ADDED_TEXT = 'Failed'
 VIBER_TEXT_FOR_NEW_SUB = 'HI!'
+# WHATSAPP
+IS_WHATS_APP_ENABLED = True
+WHATS_APP_AUTH_TOKEN = None
+WHATS_APP_ACCOUNT_SID = None
+WHATS_APP_NUMBER_SENDER = None
 # SETTINGS
 EMAIL_MALLING = 1
 GARPIX_NOTIFY_MIXIN = 'garpix_notify.mixins.notify_mixin.NotifyMixin'
