@@ -3,14 +3,15 @@ from os import path
 
 here = path.join(path.abspath(path.dirname(__file__)), 'garpix_notify')
 
-with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
+with open(path.join(here, 'README.md'), encoding='utf-8') as f:
     long_description = f.read()
 
 setup(
     name='garpix_notify',
-    version='5.6.2',
+    version='5.10.0',
     description='',
     long_description=long_description,
+    long_description_content_type='text/markdown',
     url='https://github.com/garpixcms/garpix_notify',
     author='Garpix LTD',
     author_email='info@garpix.com',
@@ -42,8 +43,9 @@ setup(
         'Pillow >= 8.2.0',
         'celery>=4.4.2',
         'redis >= 3.5.3',
-        'channels == 3.0.3',
-        'channels-redis == 3.3.1',
-        'asgiref == 3.2.10',
+        'channels == 3.0.4',
+        'channels-redis == 3.4.0',
+        'asgiref == 3.3.2',
+        'twilio == 7.10.0',
     ],
 )
