@@ -1,6 +1,5 @@
 window.onload = function(){
     let sms_url = document.querySelector('#id_sms_url_type');
-    let call_url = document.querySelector('#id_call_url_type');
 
     if (parseInt(sms_url.value) === 0) {
         document.querySelector('.field-sms_login').classList.add('admin-none');
@@ -45,17 +44,4 @@ window.onload = function(){
             document.querySelector('.field-sms_from').classList.add('admin-none');
         }
     });
-
-    call_url.addEventListener('change', (event) => {
-        if (parseInt(event.currentTarget.value) === 0) {
-            document.querySelector('.field-call_api_id').classList.remove('admin-none');
-            document.querySelector('.field-call_login').classList.add('admin-none');
-            document.querySelector('.field-call_password').classList.add('admin-none');
-        }
-        else {
-            document.querySelector('.field-call_login').classList.remove('admin-none');
-            document.querySelector('.field-call_password').classList.remove('admin-none');
-            document.querySelector('.field-call_api_id').classList.add('admin-none');
-        }
-    });
 }
\ No newline at end of file
